# coding=utf-8

import serial, time
from shtiny_list import *

# デバッグ情報表示レベル（0: 非表示）
verbose = 0

# 与えられたバイト列に対するチェックサムを計算する
def checksum_for_x(x):
	if hasattr(x, '__iter__'):
		s = sum(x)
	else:
		s = x
	return (0x10000000 - s) % 256


def Connect(ser):
	ser.write(chr(0x00))
	ret = ser.read()
	if not ret:
		return 1 # 返答がない
	elif ret == chr(0x00):
		ser.write(chr(0x55))
		ret = ser.read()
		if ret == chr(0xE6):
			return 0 # ビットレート合わせ込み完了
		elif ret == chr(0xFF):
			return 3 # エラー
		else:
			return 4 # 想定外の文字が返ってきた
	else:
		return 2 # 想定外の文字が返ってきた


def Inquiry_on_supported_devices(ser):
	checksum = 0
	# コマンド
	ser.write(cmd_code["Inquiry_on_supported_devices"])

	# レスポンス
	c = ser.read()
	if c != chr(0x30):
		raise Exception("Response is not 0x30");
	checksum += ord(c)

	# サイズ
	n = ord(ser.read())
	checksum += n

	# デバイス数
	n_device = ord(ser.read())
	checksum += n_device
	if verbose > 0:
		print "Number of devices: %d" % n_device

	for i in range(n_device):
		# 文字数
		n_char = ord(ser.read())
		checksum += n_char

		# デバイスコード
		device_code = ser.read(4)
		if verbose > 0:
			print "    Device code: %s" % device_code
		for x in device_code:
			checksum += ord(x)

		# 品名
		product_name = ser.read(n_char-4)
		if verbose > 0:
			print "    Product name: %s" % product_name
		for x in product_name:
			checksum += ord(x)

	# SUM
	checksum += ord(ser.read())
	if (checksum % 256) != 0:
		raise  Exception("Checksum error")


def Device_selection(ser, device_code):
	checksum = 0
	if len(device_code) != 4:
		raise Exception("Size of device code must be 4.")

	# コマンド
	ser.write(cmd_code["Device_selection"])
	checksum += ord(cmd_code["Device_selection"])

	# サイズ
	ser.write(chr(4))
	checksum += 4;

	# デバイスコード
	ser.write(device_code)
	for x in device_code:
		checksum += ord(x)

	# SUM
	checksum = checksum_for_x(checksum)
	ser.write(chr(checksum))

	# レスポンス
	c = ser.read()
	if c == ACK:
		if verbose > 0:
			print "Device was set to %s" % device_code
	elif c == chr(0x90):
		c = ser.read()
		print "ERROR: %s" % error_code[c]
	else:
		raise Exception("Unknown response")


def Inquiry_on_clock_modes(ser):
	checksum = 0
	# コマンド
	ser.write(cmd_code["Inquiry_on_clock_modes"])

	# レスポンス
	c = ser.read()
	if c != chr(0x31):
		raise Exception("Response is not 0x31");
	checksum += ord(c)

	# サイズ
	n = ord(ser.read())
	checksum += n

	# モード
	if verbose > 0:
		print "Selectable clock mode:"
	for i in range(n):
		mode = ord(ser.read())
		checksum += mode
		if verbose > 0:
			print "    %d" % mode
	
	# SUM
	checksum += ord(ser.read())
	if (checksum % 256) != 0:
		raise  Exception("Checksum error")


def Clock_mode_selection(ser, clock_mode):
	checksum = 0

	# コマンド
	ser.write(cmd_code["Clock-mode_selection"])
	checksum += ord(cmd_code["Clock-mode_selection"])

	# サイズ
	ser.write(chr(1))
	checksum += 1;

	# クロックモード
	ser.write(clock_mode)
	checksum += ord(clock_mode)

	# SUM
	checksum = checksum_for_x(checksum)
	ser.write(chr(checksum))

	# レスポンス
	c = ser.read()
	if c == ACK:
		if verbose > 0:
			print "Clock-mode was set to %d" % ord(clock_mode)
	elif c == chr(0x90):
		c = ser.read()
		print "ERROR: %s" % error_code[c]
	else:
		raise Exception("Unknown response")


def Inquiry_on_frequency_multipliers(ser):
	checksum = 0
	# コマンド
	ser.write(cmd_code["Inquiry_on_frequency_multipliers"])

	# レスポンス
	c = ser.read()
	if c != chr(0x32):
		raise Exception("Response is not 0x32");
	checksum += ord(c)

	# サイズ
	n = ord(ser.read())
	checksum += n

	# 周波数の種別数
	n_freq_type  = ord(ser.read())
	checksum += n_freq_type

	for i in range(n_freq_type):
		if verbose > 0:
			print "Multiplier for frequency type %d:" % (i+1)
		# 逓倍比数
		n_mul = ord(ser.read())
		checksum += n_mul

		for j in range(n_mul):
			# 逓倍比
			mul = ord(ser.read())
			checksum += mul
			if mul > 127:
				mul -= 256
			if verbose > 0:
				print "    %d" % mul

	# SUM
	checksum += ord(ser.read())
	if (checksum % 256) != 0:
		raise  Exception("Checksum error")


def Inquiry_on_operating_frequency(ser):
	checksum = 0
	# コマンド
	ser.write(cmd_code["Inquiry_on_operating_frequency"])

	# レスポンス
	c = ser.read()
	if c != chr(0x33):
		raise Exception("Response is not 0x33");
	checksum += ord(c)

	# サイズ
	n = ord(ser.read())
	checksum += n

	# 周波数の種別数
	n_freq_type  = ord(ser.read())
	checksum += n_freq_type

	for i in range(n_freq_type):
		if verbose > 0:
			print "Operating frequency for frequency type %d:" % i
		# 動作周波数最小値
		s = ser.read(2)
		for x in s:
			checksum += ord(x)
		freq = float((ord(s[0]) << 8) + ord(s[1])) / 100
		if verbose > 0:
			print "    min: %.2f MHz" % freq

		# 動作周波数最大値
		s = ser.read(2)
		for x in s:
			checksum += ord(x)
		freq = float((ord(s[0]) << 8) + ord(s[1])) / 100
		if verbose > 0:
			print "    max: %.2f MHz" % freq

	# SUM
	checksum += ord(ser.read())
	if (checksum % 256) != 0:
		raise  Exception("Checksum error")


def Inquiry_on_user_MATs(ser):
	checksum = 0
	# コマンド
	ser.write(cmd_code["Inquiry_on_user_MATs"])

	# レスポンス
	c = ser.read()
	if c != chr(0x35):
		raise Exception("Response is not 0x35");
	checksum += ord(c)

	# サイズ
	n = ord(ser.read())
	checksum += n

	# 周波数の種別数
	n_area  = ord(ser.read())
	checksum += n_area

	for i in range(n_area):
		if verbose > 0:
			print "Address for user MAT area %d:" % i
		# エリア先頭アドレス
		s = ser.read(4)
		for x in s:
			checksum += ord(x)
		address  = ord(s[0]) << 24
		address += ord(s[1]) << 16
		address += ord(s[2]) <<  8
		address += ord(s[3])
		if verbose > 0:
			print "    first: 0x%08X" % address

		# エリア最終アドレス
		s = ser.read(4)
		for x in s:
			checksum += ord(x)
		address  = ord(s[0]) << 24
		address += ord(s[1]) << 16
		address += ord(s[2]) <<  8
		address += ord(s[3])
		if verbose > 0:
			print "    last:  0x%08X" % address

	# SUM
	checksum += ord(ser.read())
	if (checksum % 256) != 0:
		raise  Exception("Checksum error")


def Inquiry_on_erasure_blocks(ser):
	checksum = 0
	# コマンド
	ser.write(cmd_code["Inquiry_on_erasure_blocks"])

	# レスポンス
	c = ser.read()
	if c != chr(0x36):
		raise Exception("Response is not 0x36");
	checksum += ord(c)

	# サイズ
	s = ser.read(2)
	for x in s:
		checksum += ord(x)
	n = (ord(s[0]) << 8) + ord(s[1])

	# 周波数の種別数
	n_block  = ord(ser.read())
	checksum += n_block

	for i in range(n_block):
		if verbose > 0:
			print "Address for erasure block %d:" % i
		# ブロック先頭アドレス
		s = ser.read(4)
		for x in s:
			checksum += ord(x)
		address  = ord(s[0]) << 24
		address += ord(s[1]) << 16
		address += ord(s[2]) <<  8
		address += ord(s[3])
		if verbose > 0:
			print "    first: 0x%08X" % address

		# ブロック最終アドレス
		s = ser.read(4)
		for x in s:
			checksum += ord(x)
		address  = ord(s[0]) << 24
		address += ord(s[1]) << 16
		address += ord(s[2]) <<  8
		address += ord(s[3])
		if verbose > 0:
			print "    last:  0x%08X" % address

	# SUM
	checksum += ord(ser.read())
	if (checksum % 256) != 0:
		raise  Exception("Checksum error")


def Inquiry_on_programming_size(ser):
	checksum = 0
	# コマンド
	ser.write(cmd_code["Inquiry_on_programming_size"])

	# レスポンス
	c = ser.read()
	if c != chr(0x37):
		raise Exception("Response is not 0x37");
	checksum += ord(c)

	# サイズ
	n = ord(ser.read())
	checksum += n

	# 書き込みサイズ
	s = ser.read(2)
	for x in s:
		checksum += ord(x)
	programming_size = (ord(s[0]) << 8) + ord(s[1])
	if verbose > 0:
		print "Programming size: %d" % programming_size

	# SUM
	checksum += ord(ser.read())
	if (checksum % 256) != 0:
		raise  Exception("Checksum error")

	return programming_size


def New_bit_rate_selection(ser, bitrate, in_freq, mul1, mul2):
	checksum = 0

	# コマンド
	ser.write(cmd_code["New_bit_rate_selection"])
	checksum += ord(cmd_code["New_bit_rate_selection"])

	# サイズ
	n = 2 + 2 + 1 + 1 + 1
	ser.write(chr(n))
	checksum += n;

	# ビットレート
	bitrate = int(bitrate / 100)
	c = (bitrate&0xFF00) >> 8
	ser.write(chr(c))
	checksum += c;
	c = bitrate&0x00FF
	ser.write(chr(c))
	checksum += c;

	# 入力周波数
	in_freq = int(in_freq * 100)
	c = (in_freq&0xFF00) >> 8
	ser.write(chr(c))
	checksum += c;
	c = in_freq&0x00FF
	ser.write(chr(c))
	checksum += c;

	# 逓倍比数
	ser.write(chr(2))
	checksum += 2;

	# 逓倍比1
	c = mul1
	if c < 0:
		c += 256
	ser.write(chr(c))
	checksum += c;

	# 逓倍比2
	c = mul2
	if c < 0:
		c += 256
	ser.write(chr(c))
	checksum += c;

	# SUM
	checksum = checksum_for_x(checksum)
	ser.write(chr(checksum))

	# レスポンス
	c = ser.read()
	if c == ACK:
		pass
	elif c == chr(0xBF):
		c = ser.read()
		print "ERROR: %s" % error_code[c]
		return
	else:
		raise Exception("Unknown response")

	# ホストのビットレートを変更する
	ser.baudrate = bitrate * 100
	time.sleep(0.01)

	# ACK送信
	ser.write(ACK)

	# レスポンス
	c = ser.read()
	if c == ACK:
		print "Bit rate was set to %d bps:" % (bitrate * 100)
		if verbose > 0:
			print "    input frequency: %.2f MHz" % (float(in_freq) / 100)
			print "    multiplier 1: %d" % mul1
			print "    multiplier 1: %d" % mul2
	else:
		raise Exception("Response is not ACK.")


def Transition_to_programming_erasure_state(ser):
	# コマンド
	ser.write(cmd_code["Transition_to_programming/erasure_state"])

	# レスポンス
	c = ser.read()
	if c == ACK:
		if verbose > 0:
			print "Enter programming/erasure state"
	elif c == chr(0xC0):
		c = ser.read()
		print "ERROR: %s" % error_code[c]
	else:
		raise Exception("Unknown response")


def Selection_of_user_MAT_programming(ser):
	# コマンド
	ser.write(cmd_code["Selection_of_user_MAT_programming"])

	# レスポンス
	c = ser.read()
	if c == ACK:
		pass
	elif c == chr(0xC3):
		c = ser.read()
		print "ERROR: %s" % error_code[c]
	else:
		raise Exception("Unknown response")


def _128_byte_programming(ser, addr, data):
	# アドレス境界チェック
	if (addr&0x7F) != 0:
		raise Exception("Address is not on a 128-byte boundary.")
	# データサイズチェック
	if len(data) > 128:
		raise Exception("Data size is too large.")
	e = chr(0xFF) * (128 - len(data))
	data += e

	b = []
	# コマンド
	b.append(ord(cmd_code["128-byte_programming"]))

	# 書き込みアドレス
	b.append((addr&0xFF000000) >> 24)
	b.append((addr&0x00FF0000) >> 16)
	b.append((addr&0x0000FF00) >> 8)
	b.append(addr&0x000000FF)

	# データ
	for x in data:
		b.append(ord(x))

	# SUM
	checksum = checksum_for_x(b)
	b.append(checksum)

	# 送信
	c = ""
	for x in b:
		c += chr(x)
	ser.write(c)

	# レスポンス
	c = ser.read()
	if c == ACK:
		pass
	elif c == chr(0xD0):
		c = ser.read()
		print "ERROR: %s" % error_code[c]
	else:
		raise Exception("Unknown response")


def end_128_byte_programming(ser):
	b = []
	# コマンド
	b.append(ord(cmd_code["128-byte_programming"]))

	# 書き込みアドレス
	b.extend([0xFF, 0xFF, 0xFF, 0xFF])

	# SUM
	checksum = checksum_for_x(b)
	b.append(checksum)

	# 送信
	c = ""
	for x in b:
		c += chr(x)
	ser.write(c)

	# レスポンス
	c = ser.read()
	if c == ACK:
		pass
	elif c == chr(0xD0):
		c = ser.read()
		print "ERROR: %s" % error_code[c]
	else:
		raise Exception("Unknown response")


#def Erasure_selection
#def Block_erasure

def Memory_read(ser, addr, size):
	checksum = 0

	b = []
	# コマンド
	b.append(ord(cmd_code["Memory_read"]))

	# サイズ
	b.append(9)

	# エリア
	b.append(0x01)

	# 読み出し先頭アドレス
	b.append((addr&0xFF000000) >> 24)
	b.append((addr&0x00FF0000) >> 16)
	b.append((addr&0x0000FF00) >> 8)
	b.append(addr&0x000000FF)

	# 読み出しサイズ
	b.append((size&0xFF000000) >> 24)
	b.append((size&0x00FF0000) >> 16)
	b.append((size&0x0000FF00) >> 8)
	b.append(size&0x000000FF)

	# SUM
	checksum += sum(b)
	checksum = checksum_for_x(checksum)
	b.append(checksum)

	# 送信
	c = ""
	for x in b:
		c += chr(x)
	ser.write(c)

	# レスポンス
	checksum = 0
	c = ser.read()
	if c == chr(0x52):
		checksum += ord(c)

		# 読み出しサイズ
		s = ser.read(4)
		for x in s:
			checksum += ord(x)
		n_read = (ord(s[0])<<24) + (ord(s[1])<<16) + (ord(s[2])<<8) + ord(s[3])
		if size != n_read:
			raise Exception("Read sizes are different: %d, %d" % (size, n_read))

		# データ
		c = ser.read(n_read)
		for x in c:
			checksum += ord(x)

		# SUM
		checksum += ord(ser.read())
		if (checksum % 256) != 0:
			raise  Exception("Checksum error")

		return c

	elif c == chr(0xD2):
		c = ser.read()
		print "ERROR: %s" % error_code[c]
	else:
		raise Exception("Unknown response")


def Sum_checking_of_user_MAT(ser):
	# コマンド
	ser.write(cmd_code["Sum_checking_of_user_MAT"])

	checksum = 0
	# レスポンス
	c = ser.read()
	if c != chr(0x5B):
		raise Exception("Response is not 0x5B");
	checksum += ord(c)

	# サイズ
	n = ord(ser.read())
	if n != 4:
		raise Exception("Size must be 4.")
	checksum += n

	# マットのチェックサム
	s = ser.read(4)
	for x in s:
		checksum += ord(x)
	checksum_mat  = ord(s[0]) << 24
	checksum_mat += ord(s[1]) << 16
	checksum_mat += ord(s[2]) <<  8
	checksum_mat += ord(s[3])

	# SUM
	checksum += ord(ser.read())
	if (checksum % 256) != 0:
		raise  Exception("Checksum error")

	return checksum_mat


def Blank_checking_of_user_MAT(ser):
	# コマンド
	ser.write(cmd_code["Blank_checking_of_user_MAT"])

	# レスポンス
	c = ser.read()
	if c == ACK:
		pass
	elif c == chr(0xCD):
		c = ser.read()
		print "ERROR: %s" % error_code[c]
	else:
		raise Exception("Unknown response")


def Inquiry_on_boot_program_state(ser):
	# コマンド
	ser.write(cmd_code["Inquiry_on_boot_program_state"])

	checksum = 0
	# レスポンス
	c = ser.read()
	if c != chr(0x5F):
		raise Exception("Response is not 0x5F");
	checksum += ord(c)

	# サイズ
	n = ord(ser.read())
	if n != 2:
		raise Exception("Size must be 4.")
	checksum += n

	# STATUS
	c = ser.read()
	status = ord(c)
	print "Status: %s" % status_code[c]
	checksum += status

	# ERROR
	c = ser.read()
	err = ord(c)
	print "Error: %d" % err
	checksum += err

	# SUM
	checksum += ord(ser.read())
	if (checksum % 256) != 0:
		raise  Exception("Checksum error")
	
	return (status, err)

