#!/usr/bin/env python
# coding=utf-8

import sys, optparse, datetime
import shtiny_serial, shtiny_cmd, shtiny_data, mot

################################################################################
# 下記を決め打ちしている
# デバイスは SH-7125
device = '1i51'
# クリスタルは12MHz
xtal = 12 # [MHz]
mul1 = 4 # メイン周波数逓倍比
mul2 = 2 # 周辺周波数逓倍比
# FLASHの大きさは128k bytes
flash_size = 128 * 1024
################################################################################

def print_time(t):
	seconds = t.days * 3600 * 24 + t.seconds + t.microseconds * 1.E-6
	print "        time: %.3f s" % seconds

try:
	# バージョン情報
	version="150210py"
	print "shtiny_writer (version: " + version + ")"


	# オプションの解析
	parser = optparse.OptionParser()

	parser.add_option(
		"-p", "--port",
		action="store",
		dest="port",
		default="",
		help="set serial port")
	parser.add_option(
		"-r", "--baudrate",
		action="store",
		dest="baudrate",
		default=57600,
		help="set baudrate")
	parser.add_option(
		"-c", "--compare",
		action="store_true",
		dest="compare_flag",
		default=False,
		help="compare written data with original mot data")
	parser.add_option(
		"-s", "--checksum",
		action="store_true",
		dest="checksum_flag",
		default=False,
		help="compare checksums of sent data and flash data")
	parser.add_option(
		"-t", "--time",
		action="store_true",
		dest="time_flag",
		default=False,
		help="print time after the start of connection with a device")
	parser.add_option(
		"-v", "--verbose",
		action="store_true",
		dest="verbose_flag",
		default=False,
		help="print detailed information for debug")

	(options, args) = parser.parse_args()
	if not args:
		print "no mot file"
		sys.exit(1)


	if options.verbose_flag:
		shtiny_cmd.verbose = True


	#######################################################################
	# 接続
	#######################################################################
	# ポートを開く
	ser = shtiny_serial.open_port(options.port, 19200)

	# shtinyへ接続
	shtiny_serial.connect_to_device(ser)

	# 時間計測初期値
	t0 = datetime.datetime.now()
	if options.time_flag:
		print_time(datetime.timedelta(0))

	#######################################################################
	# 設定
	#「コマンドの順序」(ハードウェアマニュアル Rev.5.00, 日本語, p.17-57)
	#######################################################################
	# 1.
	shtiny_cmd.Inquiry_on_supported_devices(ser)

	# 2.
	shtiny_cmd.Device_selection(ser, device)

	# 3.
	shtiny_cmd.Inquiry_on_clock_modes(ser)

	# 4.
	shtiny_cmd.Clock_mode_selection(ser, chr(0x00))

	# 5.
	shtiny_cmd.Inquiry_on_frequency_multipliers(ser)
	shtiny_cmd.Inquiry_on_operating_frequency(ser)

	# 6.
	shtiny_cmd.New_bit_rate_selection(ser, int(options.baudrate), xtal, mul1, mul2)

	# 7.
	shtiny_cmd.Inquiry_on_user_MATs(ser)
	shtiny_cmd.Inquiry_on_erasure_blocks(ser)
	programming_size = shtiny_cmd.Inquiry_on_programming_size(ser)

	# 8.
	shtiny_cmd.Transition_to_programming_erasure_state(ser)


	#######################################################################
	# 書き込み
	#######################################################################
#	shtiny_cmd.Inquiry_on_boot_program_state(ser)
	shtiny_cmd.Blank_checking_of_user_MAT(ser)

	if options.time_flag:
		print_time(datetime.datetime.now() - t0)

	for filename in args:
		# motファイルからデータ抽出
		(addr, data, start_addr) = mot.analyze_mot_file(filename)

		# データレコード再構成
		(addr, data) = mot.reconstruct_records(addr, data, programming_size)

		if options.time_flag:
			print_time(datetime.datetime.now() - t0)

		# 書き込み
		shtiny_data.write_data(ser, addr, data)

		if options.time_flag:
			print_time(datetime.datetime.now() - t0)

		# 書き込んだデータとオリジナルの比較
		if options.compare_flag:
			shtiny_data.compare_data(ser, addr, data)

			if options.time_flag:
				print_time(datetime.datetime.now() - t0)

		# 書き込んだデータとオリジナルのチェックサムの比較
		if options.checksum_flag:
			shtiny_data.compare_checksum(ser, data, flash_size)

			if options.time_flag:
				print_time(datetime.datetime.now() - t0)

		print

	# ポートを閉じる
	shtiny_serial.close_port(ser)

except KeyboardInterrupt:
	print
	sys.exit(1)

